def _parse_env_args_as_dict(env_args):
    import ast
    """
    We assume the env args are represented as a dictionary string, i.e.
    {name1:value1,name2:value2,...}
    """
    if not env_args: env_args = '{}'
    d = ast.literal_eval(env_args)
    if isinstance(d, dict):
        return d
    raise ValueError("Expected a dictionary string as env_args!")


def _create_env_racetrack(args):
    import safety_gymnasium
    from racetrackgym.argument_parser import Racetrack_parser
    from racetrackgym.wrapper import SafetyGymnasiumRaceTrackEnv
    
    # NOTE: racetrackgym has a custom parser, use it instead
    rt_args = Racetrack_parser().parse(args.env_args.split())
    args.env_args = rt_args.__dict__

    safety_gymnasium.register(id=args.env_id, entry_point=SafetyGymnasiumRaceTrackEnv)
    env = safety_gymnasium.make(id=args.env_id, rt_args=rt_args)
    return env


def _create_env_safetygym(args):
    import safety_gymnasium
    from safety_gymnasium.bases.base_task import LidarConf
    
    # NOTE: we dont have a safety gymnasium argument parser, hence expect arguments as a dictionary string
    sg_args = _parse_env_args_as_dict(args.env_args)
    args.env_args = sg_args.copy()

    # allow changing the lidar via args
    if 'lidar' in sg_args:
        lidar_config = dict()
        for key, val in sg_args['lidar'].items():
            lidar_config[key] = val
        
        # Wrap init function with new default values
        def new_init(self, **kwargs):
            for key, val in lidar_config.items():
                if key not in kwargs:
                    kwargs[key] = val
            return self.__init_original__(**kwargs)
        
        LidarConf.__init_original__ = LidarConf.__init__
        LidarConf.__init__ = new_init
        del sg_args['lidar']

    env = safety_gymnasium.make(id=args.env_id)
    return env


def create_env(args):
    name = args.env_id
    if name == "RaceTrack":
        return _create_env_racetrack(args)
    elif name.startswith("Safety"):
        return _create_env_safetygym(args)
    else:
        raise RuntimeError("Unkonwn environment: " + name)
