import numpy as np

class Statistics:
    def __init__(self, writer):
        self.writer = writer

        # Total training statistics
        self.total_train_episodes = 0
        self.total_train_steps = 0
        self.total_train_unsafe = 0
        self.total_updates = 0

        # Store avg unsafe episodes for CSC
        self.train_unsafe_avg = 0
    
    def log_train_tensorboard(self, episode_steps, episode_return, episode_cost):
        name_prefix = f"train/"
        self.log_tensorboard(name_prefix + 'steps', episode_steps, self.total_train_steps)
        self.log_tensorboard(name_prefix + 'returns', episode_return, self.total_train_steps)
        self.log_tensorboard(name_prefix + 'costs', episode_cost, self.total_train_steps)
    
    def log_test_tensorboard(self, avg_steps, avg_return, avg_cost, avg_unsafe, shielded=True):
        name_prefix = f"test/{'shielded' if shielded else 'unshielded'}/"
        self.log_tensorboard(name_prefix + 'avg_steps', avg_steps, self.total_train_steps)
        self.log_tensorboard(name_prefix + 'avg_return', avg_return, self.total_train_steps)
        self.log_tensorboard(name_prefix + 'avg_cost', avg_cost, self.total_train_steps)
        self.log_tensorboard(name_prefix + 'avg_unsafe', avg_unsafe, self.total_train_steps)
    
    def log_update_tensorboard(self, data:dict):
        for k, v in data.items():
            name = f"update/{k}"
            self.writer.add_scalar(name, v, self.total_updates)

    def log_tensorboard(self, name, value, step):
        self.writer.add_scalar(name, value, step)